﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class GameManager : MonoBehaviour {

	protected float gameTimer;
	protected float spawnTimer;

	protected int playerScore;
	protected bool gameOver = false;

	protected MobSpawner mobSpawner;
	protected GameObject firstPersonController;
	protected GameObject mainCamera;

	public GameObject explosion;

	protected AudioSource soundGoodHit;
	protected AudioSource soundBadHit;

	// Use this for initialization
	void Start () {
		Utils.Init(this);
		this.gameTimer = 60.0f;
		this.spawnTimer = 4.0f;

		this.playerScore = 0;

		this.mobSpawner = this.gameObject.GetComponent<MobSpawner>();
		this.firstPersonController = GameObject.Find("First Person Controller");
		this.mainCamera = GameObject.Find("Main Camera");

		AudioSource[] aAudioSources = this.gameObject.GetComponents<AudioSource>();
		this.soundGoodHit = aAudioSources[0];
		this.soundBadHit = aAudioSources[1];

		Screen.showCursor = false;
	}
	
	// Update is called once per frame
	void Update () {

		if(this.gameOver){
			return;
		}

		this.gameTimer -= Time.deltaTime;
		this.spawnTimer -= Time.deltaTime;

		// Game over
		if(this.gameTimer <= 0){
			this.gameOver = true;
			this.mobSpawner.enabled = false;
			this.firstPersonController.GetComponent<CharacterMotor>().enabled = false;
			this.firstPersonController.GetComponent<MouseLook>().enabled = false;
			this.firstPersonController.GetComponent<FPSInputController>().enabled = false;
			this.mainCamera.GetComponent<MouseLook>().enabled = false;
			if(this.mainCamera.GetComponent<AltControls>()){
				this.mainCamera.GetComponent<AltControls>().enabled = false;
			}else{
				this.mainCamera.GetComponent<Controls>().enabled = false;
			}
			Screen.showCursor = true;
		}

		// Spawn mob
		if(this.spawnTimer <= 0){
			int iEnumLength = System.Enum.GetValues(typeof(LightColor)).Length;
			int iEnumId = Mathf.FloorToInt(Random.value * iEnumLength);
			LightColor eLightColor = (LightColor)System.Enum.GetValues(typeof(LightColor)).GetValue(iEnumId);
			this.mobSpawner.SpawnMob(eLightColor);
			this.spawnTimer = 2 + Random.value * 4;
		}

	}

	public void GoodHitSpirit(LightSpirit oLightSpirit){
		this.playerScore += Mathf.CeilToInt(this.gameTimer);
		this.gameTimer += 5;
		GameObject oExplosion = (GameObject)GameObject.Instantiate(this.explosion);
		oExplosion.transform.position = oLightSpirit.gameObject.transform.position;
		oExplosion.particleSystem.startColor = (Color)Utils.lightColorsColors[oLightSpirit.GetLightColor()];
		this.soundGoodHit.Play();
		GameObject.Destroy(oLightSpirit.gameObject);
		GameObject.Destroy(oExplosion, 2);
	}

	public void BadHitSpirit(LightBall oLightBall){
		GameObject oExplosion = (GameObject)GameObject.Instantiate(this.explosion);
		oExplosion.transform.position = oLightBall.gameObject.transform.position;
		oExplosion.particleSystem.startColor = (Color)Utils.lightColorsColors[oLightBall.GetLightColor()];
		this.soundBadHit.Play();
		GameObject.Destroy(oExplosion, 2);
	}

	public float GetGameTimer(){
		return this.gameTimer;
	}
	
	public bool GetGameOver(){
		return this.gameOver;
	}

	public int GetPlayerScore(){
		return this.playerScore;
	}
}
